"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Res = exports.Req = exports.Body = exports.Param = exports.Patch = exports.Delete = exports.Put = exports.Post = exports.Get = exports.Controller = void 0;
require("reflect-metadata");
const constants_1 = require("./constants");
const interfaces_1 = require("./interfaces");
const parametersDecoratorFactory = (type, name) => {
    return (target, key) => {
        const lateParams = Reflect.getMetadata(constants_1.MetadataTypes.Params, target.constructor) || {};
        const params = {
            [key]: [{ type, name }, ...(lateParams[key] || [])],
        };
        Reflect.defineMetadata(constants_1.MetadataTypes.Params, params, target.constructor);
    };
};
const methodDecoratorFactory = (type, path = '/') => {
    return (target, propertyKey) => {
        Reflect.defineMetadata(constants_1.MetadataTypes.Routes, [
            ...(Reflect.getMetadata(constants_1.MetadataTypes.Routes, target.constructor) ||
                []),
            { requestMethod: type, path, method: propertyKey },
        ], target.constructor);
    };
};
const Controller = (path = '') => {
    return (target) => {
        constants_1.controllers.add(target);
        Reflect.defineMetadata(constants_1.MetadataTypes.Prefix, path, target);
    };
};
exports.Controller = Controller;
const Get = (path) => methodDecoratorFactory('get', path);
exports.Get = Get;
const Post = (path) => methodDecoratorFactory('post', path);
exports.Post = Post;
const Put = (path) => methodDecoratorFactory('put', path);
exports.Put = Put;
const Delete = (path) => methodDecoratorFactory('delete', path);
exports.Delete = Delete;
const Patch = (path) => methodDecoratorFactory('patch', path);
exports.Patch = Patch;
const Param = (name) => parametersDecoratorFactory(interfaces_1.ParametersType.Param, name);
exports.Param = Param;
const Body = (name) => parametersDecoratorFactory(interfaces_1.ParametersType.Body, name);
exports.Body = Body;
const Req = () => parametersDecoratorFactory(interfaces_1.ParametersType.Req);
exports.Req = Req;
const Res = () => parametersDecoratorFactory(interfaces_1.ParametersType.Res);
exports.Res = Res;
