"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypediExpressServer = void 0;
const express_1 = __importStar(require("express"));
const body_parser_1 = __importDefault(require("body-parser"));
const constants_1 = require("./constants");
const interfaces_1 = require("./interfaces");
class TypediExpressServer {
    constructor(container) {
        this.container = container;
        this.app = (0, express_1.default)();
    }
    build() {
        this.registerControllers();
        return this.app;
    }
    registerControllers() {
        constants_1.controllers.forEach((controller) => {
            const restControllers = this.container.has(constants_1.ContainerTypes.Controller)
                ? this.container.get(constants_1.ContainerTypes.Controller)
                : [];
            this.container.set(constants_1.ContainerTypes.Controller, restControllers.concat(controller));
        });
        this.container
            .get(constants_1.ContainerTypes.Controller)
            .forEach((Controller) => {
            const instance = this.container.get(Controller);
            const router = (0, express_1.Router)();
            const prefix = Reflect.getMetadata(constants_1.MetadataTypes.Prefix, Controller);
            const routes = Reflect.getMetadata(constants_1.MetadataTypes.Routes, Controller);
            const controllerParams = Reflect.getMetadata(constants_1.MetadataTypes.Params, Controller);
            routes.forEach((route) => {
                const params = controllerParams[route.method];
                router[route.requestMethod]('/' + route.path, (req, res) => __awaiter(this, void 0, void 0, function* () {
                    const paramsMapping = this.extractParams(params, req, res);
                    const result = yield instance[route.method](...paramsMapping);
                    res.send(result);
                }));
            });
            this.app.use(body_parser_1.default.json());
            this.app.use('/' + prefix, router);
        });
    }
    extractParams(params, req, res) {
        return params.map((key) => {
            switch (key.type) {
                case interfaces_1.ParametersType.Param:
                    return key.name ? req.params[key.name] : req.params;
                case interfaces_1.ParametersType.Req:
                    return req;
                case interfaces_1.ParametersType.Res:
                    return res;
                case interfaces_1.ParametersType.Body:
                    return req.body;
                default:
                    throw new Error('Unexpected metadata type');
            }
        });
    }
}
exports.TypediExpressServer = TypediExpressServer;
